/**
 * $Id: AffirmCreditCard.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package com.inxar.affirm;

import org.inxar.affirm.*;

/**
 * <code>Affirmation</code> implementation which checks that the value
 * associated with a certain key is a valid credit card number as
 * checked by the Luhn check digit algorithm.  
 */
public class AffirmCreditCard
    extends AbstractAffirmation
{
    public AffirmCreditCard(Object key, String msg, boolean isFatal, String pattern) throws Exception
    {
    	super(key, msg, isFatal);
    	this.luhn = new Luhn();
    }

    public AffirmCreditCard(Object key, boolean isFatal, String pattern) throws Exception
    {
    	this(key, "The given credit card is invalid", isFatal, pattern);
    }

    public AffirmCreditCard(Object key, String pattern) throws Exception
    {
    	this(key, "The given credit card is invalid", true, pattern);
    }

    public Object affirm(Object val) throws AffirmationException
    {
        // already ok?
        if (val instanceof String) {
	    int[] cc = luhn.prep((String)val);
	    if (luhn.isValid(cc))
		return cc;
	    else
		throw new AffirmationException(msg, isFatal);
       	} else {
	    throw new AffirmationException(msg, isFatal);
	}
    }

    private Luhn luhn;

    private static class Luhn
    {
	Luhn()
	{
	}

	/**
	 * Validates a credit card string for numeric validity.
	 * Therefore, this method will tell you whether the user has
	 * input the number incorrectly, not whether their account is
	 * actually valid.<P/>
	 *
	 * Method takes a string argument.  This string is stripped of
	 * any non-numeric characters and is validated accoring to the
	 * LUHN formula (mod10).<P/>
	 *
	 * This method is viable for the following card types: MC,
	 *  VISA, AMEX, Diners, Discover, enRoute, JCB.<P/>
	 *
	 * @param String the raw credit card string, like '2848 9788
	 * 1222 9873' or '2848978812229873' (delimiter is arbitrary).
	 * @return boolean true if the card is valid, false otherwise.  
	 */
	public boolean isValid(String raw)
	{
	    // abort or punt
	    return (raw == null) ? false : isValid(prep(raw));
	}

	/**
	 * Does the actual work.  Would use bytes for minor space
	 * optimization, but implicit numeric promotion makes this
	 * wasteful.  
	 */
	public boolean isValid(int[] cc)
	{
	    // cache the length
	    int len = cc.length;

	    // all the credit cards we check should be between 13 and 19 digits.
	    if (len<13 || len>19)
		return false;

	    // the register which will be mod 10'd
	    int sum = 0;

	    // the multiplication factor.  We multiply every other
	    // digit by 2 (not the first one).
	    int mul = 1;

	    // step over the card number string *backwards*
	    for (int i = len-1; i >= 0; i--) {
		// grab this for posterity
		int digit = cc[i];
    	     	// multiply by the factor (1 | 2) and move mul up or down.
    	     	// note alternating side-effect
		digit *= (mul == 1) ? mul++ : mul--;
		// add the remainder+1 or just the digit depending on
		// if this digit is 10 or more.
		sum += (digit >= 10) ? (digit % 10) + 1 : digit;
	    }

	    // reject the zero case
	    if (sum == 0)
		return false;

	    else
		// valid credit cards will have a sum that is
		// divisible by 10.
		return (sum % 10) == 0;
	}

	/**
	 * Returns the numeric characters in the string according to
	 * Character.isDigit(char) as a new string.
	 */
	public int[] prep(String raw)
	{

	    // cache the length
	    int len = raw.length();
	    // a char array
	    int[] tmp = new int[len];
	    // our array indexer
	    int count = 0;

	    // walk over string
	    for (int i=0; i<len; i++) {
		// cache the char
		char c = raw.charAt(i);
		// test if it is a digit
		if (Character.isDigit(c))
		    // slot the char in the next spot
		    tmp[count++] = Character.digit(c, 10);
	    }

	    // check for change
	    if (count == len) {
		// no further work necessary
		return tmp;

		// else alloc new array of correct length
	    } else {
		// make new array with the populated chars.
		int[] result = new int[count];

		// native copy
		System.arraycopy(tmp, 0, result, 0, count);

		// and done
		return result;
	    }
	}
    }
}


