/**
 * $Id: XMLProclamation.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package com.inxar.affirm;

import org.inxar.affirm.*;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * Concrete implementation of Proclamation that reads an XML document
 * to compile the list of affirmations.  This class requires
 * <code>com.sun.xml.tree.XmlDocument</code>.
 */
public class XMLProclamation
    extends AbstractProclamation
{
    public XMLProclamation(String uri) throws Exception
    {
	this( DOM.getDocument(uri) );
    }

    public XMLProclamation(Document doc) throws Exception
    {
    	this.map = new java.util.Hashtable();

        // make the document and get the default element
        this.element = doc.getDocumentElement();

	//System.out.println(element);

	// grab the database nodes
        NodeList nodes = element.getChildNodes();
	// iterate them
        for (int i=0; i<nodes.getLength(); i++) {
	    //System.out.println(nodes.item(i));
	    Node node = nodes.item(i);
	    if (node.getNodeType() == Node.ELEMENT_NODE)
		affirm( create((Element)node) );
	    //else 
	    //System.out.println("There is a node that's not an element here: '"+node+"'");
        }
    }

    protected Affirmation create(Element e) throws Exception
    {
	// get the name of the element
	String name = e.getTagName();
	// fetch the key by perhaps transforming it first
	Object key = newInstance (DOM.getString("key-type", e),
				  DOM.getString("key", e));

	// and finally the fatal flag
	boolean isFatal = DOM.getBoolean("isFatal", e);

	// now do a long string switch
	if (name.equals("affirm-exists")) 	return new AffirmExists(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-int")) 	return new AffirmInt(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-string")) 	return new AffirmString(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-boolean")) return new AffirmBoolean(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-byte")) 	return new AffirmByte(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-short")) 	return new AffirmShort(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-long")) 	return new AffirmLong(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-float")) 	return new AffirmFloat(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-double")) 	return new AffirmDouble(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-char")) 	return new AffirmChar(key, DOM.getText(e), isFatal);
	else if (name.equals("affirm-type"))
	    return new AffirmType(key, DOM.getText(e), isFatal, Class.forName(DOM.getString("class", e)));
	else if (name.equals("affirm-length"))
	    return new AffirmStringLength(key, DOM.getText(e), isFatal, DOM.getInt("min", e), DOM.getInt("max", e));
	else if (name.equals("affirm-int-range"))
	    return new AffirmIntegralRange(key, DOM.getText(e), isFatal, DOM.getInt("lo", e), DOM.getInt("hi", e));
	else if (name.equals("affirm-double-range"))
	    return new AffirmDoubleRange(key, DOM.getText(e), isFatal, DOM.getDouble("lo", e), DOM.getDouble("hi", e));
	else if (name.equals("affirm"))
	    return createGeneral(e, key, isFatal);
        else
	    throw new Exception("Unrecognized element: "+name);
    }

    protected Affirmation createGeneral(Element e, Object key, boolean isFatal) throws Exception
    {
        // get the class name to invoke
        String className = DOM.getString("class", e);
        // get the detail message
	NodeList list = e.getElementsByTagName("detail");
	// use the first one
        String msg = DOM.getText((Element)list.item(0));
        // iterate all the arguments
	// grab the database nodes
        NodeList nodes = element.getElementsByTagName("arg");
	// make the vector big enough
	Class[] types = new Class[nodes.getLength() + 3];
	Object[] args = new Object[nodes.getLength() + 3];
	args[0] = key; types[0] = Object.class;
	args[1] = msg; types[1] = String.class;
	args[2] = new Boolean(isFatal); types[2] = boolean.class;

	Element elem = null;
	// iterate them
        for (int i=3; i<args.length; i++) {
	    // get the elem
	    elem = (Element)nodes.item(i - 3);
	    String type = DOM.getString("type", elem);
	    String value = DOM.getString("value", elem);
	    // type the arg
	    args[i] = newInstance(type, value);
	    // set the types
	    types[i] = newClass(type);
        }

        //System.out.println("args are:");
        //for (int i=0; i<args.length; i++)
        //	System.out.println(types[i]+" "+args[i]);

        // fetch the ctor and instantiate
        return (Affirmation)Class.forName(className)
	    .getConstructor(types)
	    .newInstance(args);
    }

    protected Object newInstance(String type, String value) throws Exception
    {
	if (type.equals("String")) {
	    return value;
	} else if (type.equals("boolean")) {
	    if (value.equalsIgnoreCase("true"))
		return Boolean.TRUE;
	    else if (value.equalsIgnoreCase("false"))
		return Boolean.FALSE;
	    else
		throw new Exception("Only allow 'true' or 'false' for boolean");
	} else if (type.equals("byte")) {
	    return new Byte(value);
	} else if (type.equals("short")) {
	    return new Short(value);
	} else if (type.equals("int")) {
	    return new Integer(value);
	} else if (type.equals("long")) {
	    return new Long(value);
	} else if (type.equals("float")) {
	    return new Float(value);
	} else if (type.equals("double")) {
	    return new Double(value);
	} else if (type.equals("char")) {
	    return new Character(value.charAt(0));
	}
	throw new Exception("Unrecognized type: '"+type+"'");
    }

    protected Class newClass(String type) throws Exception
    {
	if (type.equals("String")) return String.class;
	else if (type.equals("boolean")) return boolean.class;
	else if (type.equals("byte")) return byte.class;
	else if (type.equals("short")) return short.class;
	else if (type.equals("int")) return int.class;
	else if (type.equals("long")) return long.class;
	else if (type.equals("float")) return float.class;
	else if (type.equals("double")) return double.class;
	else if (type.equals("char")) return char.class;
	else throw new Exception("Unrecognized type: '"+type+"'");
    }

    Element element;
}


