/**
 * $Id: Affirmation.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package org.inxar.affirm;

/**
 * The <code>Affirmation</code> interface is a central concept to the
 * design of this package as it encapsulates a single unit of work
 * towards the verification of values given under predetermined keys
 * in an <code>Input</code> object.  <code>Affirmation</code>
 * implementations make up the atomic units in a
 * <code>Proclamation</code> and are ultimately responsible for the
 * decisions: 'is this piece of data valid?'.  It is crucial to note,
 * however, that (in general) <code>Affirmation</code> implementations
 * not *only* are responsible for deciding whether a certain unit of
 * data is valid, it also *may* be responsible for putting that unit
 * of data into a more useable form.  Thus, <code>Affirmation</code>s
 * are not just passive observers -- they may do work to transform the
 * information from one representation to another.  If so, it returns
 * this <code>Object</code> as the return value of the
 * <code>affirm()</code> method.  For example -- assume you have an
 * implementation that has the task of determining whether a given
 * input is a valid boolean.  The actual raw input may turn out to be
 * a string <code>"true"</code> or <code>"yes"</code>.  Thus, the
 * <code>Affirmation</code> may decide (implementation-dependent) to
 * allow both these values as valid and under both cases return
 * <code>Boolean.TRUE</code>.  <P> It is up to the developer to
 * document and up to the user to understand what each specific
 * <code>Affirmation</code> implementation is designed to accomplish.  
 */
public interface Affirmation
{
    /**
     * Returns the key for which this affirmation targets.
     */
    Object getKey();

    /**
     * Evaluates the given <code>Input</code> for validity.  If valid,
     * the method should return with an Object to be subsequently
     * included in a <code>Datum</code> and <code>Data</code>
     * container.  If invalid, an <code>AffirmationException</code> is
     * thrown which carries out the error message and isFatal flag.  
     */
    Object affirm(Object input) throws AffirmationException;

    /**
     * Returns true if upon negative affirmation all subsequent
     * processing of this key should be halted.  
     */
    boolean isFatal();
}


