/**
 * $Id: Proclamation.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package org.inxar.affirm;

/**
 * A <code>Proclamation</code> is a set of affirmations applied
 * sequentially to an <code>Input</code>.  The overall process of
 * verifying data from some input is captured and controlled by this
 * Object.  The <code>proclaim(Input)</code> method takes an
 * <code>Input</code> object, applies each <code>Affirmation</code> to
 * that <code>Input</code> which will either (1) complete successfully
 * upon which the method returns with the verified information in a
 * <code>Data</code> object or (2) one or more Affirmations failed and
 * control transfers to the Java exception handling mechanism.  The
 * <code>ProclamationException</code> instantiated by this process
 * both (1) acts to notify that the input is not completely valid and
 * (2) carries out the <code>Data</code> and <code>Errata</code>
 * objects to the caller (rescue).  The <code>Data</code> and
 * <code>Errata</code> objects are public fields on the
 * <code>ProclamationException</code> object and are 'guaranteed' (by
 * contract) to be non-null (though they may be empty).  The
 * information from the <code>Input</code> is partitioned into 'good'
 * data (in the <code>Data</code> object) and 'bad' data (in the
 * <code>Errata</code> object.
 *
 * <code>Proclamation</code> implementations should be designed to be
 * non-mutable.  Therefore, once the object is constructed it should
 * not change such that it may handle multiple threads simultaneously.
 * */
public interface Proclamation
{
    /**
     * Returns a sequence over the set of keys defined for this
     * proclamation.  Therefore, given an <code>Input</code>, the keys
     * given by this enumeration will be those attempted to be
     * affirmed.  The objects returned by the
     * <code>java.util.Enumeration.nextElement()</code> method are
     * those objects which acts as expected keys in the input.
     * Empirically, the most common type of key is a
     * <code>String</code>.
     */
    java.util.Enumeration keys();

    /**
     * Each key in a <code>Proclamation</code> may have several
     * <code>Affirmation</code> objects defined upon it which are
     * evaluated sequentially during a <code>proclaim()</code>
     * invocation.  This method will return <code>Affirmation</code>
     * objects upon each valid call to
     * <code>java.util.Enumeration.nextElement()</code>.  
     */
    java.util.Enumeration enumerate(Object key);

    /**
     * The <code>proclaim()</code> method is used to trigger
     * verification of the given <code>Input</code>.  If the
     * <code>Affirmation</code> objects applied to the
     * <code>Input</code> return successfully, no exception will be
     * thrown.  However, if any fail it will be recorded in an
     * <code>Errata</code> object and returned to the caller via the
     * exception handling mechanism, carried on the back of the
     * <code>ProclamationException</code> object.  
     */
    Data proclaim(Input input) throws ProclamationException;
}


